C   This file contains Fortran 90 subroutines that help solve the various 
C   DGE models considered in the First Part of the book.
C
C   Contents:
C
C    SolveVI3: Computes the policy function for the simple stochastic,
C              infinite horizon Ramsey model in which the productivity
C              shock follows a Markov chain.
C
C
C  --------------------------  SolveVI -------------------------------------
C
C   Purpose: Computes the policy function for a simple stochastic, infinite
C            horizon Ramsey model. In this model, capital K is the single
C            variable) factor of production and depreciates at the constant
C            rate delta (must be in the interval (0,1]). Output is given
C            by ZF(K). The production function F is strictly increasing in K,
C            the productivity shock follows a Markov chain with m elemente,
C            the utility function u(C) is strictly concave in C=ZF(K)+(1-delta)*K-K'. 
C             
C   Algorithm: The policy function is computed on a grid of n points,
C              supplied by the user in the column vector grid. The
C              method employed is iteration over the value function.
C
C              The iterations exploid the fact that the policy as well as
C              the value function are monotonically increasing in the
C              capital stock.
C
C   Usage:  Call SolveVI(u,beta,grid,z,p,n,m,v0,g);
C
C   Input :  u= a pointer to the real(8) function that returns
C                 u(ZF(K)+(1-delta)*K-K') for a triple (Z,K,K') of points
C                 from the vector z and the grid.
C
C          beta:= Real(8) the discount factor from the agents life-time utility function.
C
C          grid:= n times 1 Real(8) column vector with elements in ascending order that
C                 represent valid stocks of the agent's capital stock.
C
C             z:= m x 1 Real(8) column vector that stores the valid realizations of
C                       Z
C
C             p:= m x m Real(8) transition matrix.
C
C             n:= Integer(4) the number of grid points
C             m:= Integer(4) the number of elements of z
C
C            v0:= n x m Real(8) matrix, the initial value function
C
C  Output:  g  := n x m Real(8) matrix. If i=1, ..., n is the
C                 index of the a point from grid, and j the index of
C                 a valid realization of the productivity shock,
C                 the element g(i,j) holds the index of next period's optimal capital stock,
C                 i.e., if K'=g(Z,K), g the policy function, then K'=grid[K[i,j]];
C
C ------------------------------------------------------------------------------------ 

  
      Subroutine SolveVI(u,beta,grid,z,p,n,m,v0,g)
 
      use DFLIB
      use IMSLF90 

      Integer(4) N, M, I, J, KS, Tmax, dg, t, nof0, stopafter
	Integer(4) kmin,kmax,ku,ko
      Integer(4) g(n,m), g1(n,m), g2(n,m), mini(m), maxi(m)
	Real(8) u, beta, grid(n), z(m), p(m,m), v0(1:n,1:n)
	Real(8) v1(n,m), v2(n,m), w(3), dv, epsilon, eps
      
	Common /DGE/ epsilon, stopafter

    	TYPE (rccoord) curpos

	External u
   
      Tmax=1000          ! maximum number of iterations 
	if (epsilon   .eq. 0.0) epsilon=0.01
	if (stopafter .eq. 0) stopafter=30

      eps=(1-beta)*epsilon
      nof0=0

C  Step 1: Initialize 
      CALL SETTEXTPOSITION(2,2,curpos)
	WRITE(10,"(A)") 'Compute Policy Function'      

      do j=1,m
        do i=1,n
	     g1(i,j)=i
C           v1(i,j)=u(z(j),grid(i),0)  ! the household consumes his entire production 
        end do   
      end do
      v1=v0
      dg=5
	dv=1

C Step 2: Iterate over the value function 
      
	t=1
C      do while ((t .lt. Tmax) .and. (dg .gt. 1) .and. (dv .gt. 1.0e-7))
      do while ((t .lt. Tmax) .and. (dv .gt. eps) .and. 
     + (nof0 .lt. stopafter))
C      do while ((t .lt. Tmax) .and. (dv .gt. eps))
      do j=1,m
      ks=1
        do i=1,n
	    kmin=ks
		kmax=n
		do while  ((kmax-kmin) .gt. 2)
	       ku=(kmax+kmin)/2
	       ko=ku+1
             w(1)=u(z(j),grid(i),grid(ku))+
     +            beta*Dot_Product(p(j,1:m),v1(ku,1:m))
	       w(2)=u(z(j),grid(i),grid(ko))+
     +            beta*Dot_Product(p(j,1:m),v1(ko,1:m))
	       if (w(2) .gt. w(1)) then
		      kmin=ku
             else
	          kmax=ko
	       endif
	     end do
	     w(1)=u(z(j),grid(i),grid(kmin))+
     +          beta*Dot_Product(p(j,1:m),v1(kmin,1:m))
           if (kmax .gt. kmin) then
		    w(2)=u(z(j),grid(i),grid(kmin+1))+
     +             beta*Dot_Product(p(j,1:m),v1(kmin+1,1:m))
           else
	        w(2)=w(1)
	     endif
	     w(3)=u(z(j),grid(i),grid(kmax))+
     +          beta*Dot_Product(p(j,1:m),v1(kmax,1:m))
           ks=IDMAX(3,w,1)		
          v2(i,j)=w(ks)
		     ks=kmin+ks-1    
          g2(i,j)=ks   
        end do  
      end do

      dg=Count(g2 .ne.g1) ! convergence criterium: g2==g1, all elements 
	if (dg .eq. 0) nof0=nof0+1
	if ((nof0 .gt. 0) .and. (dg .gt. 0)) nof0=0

	dv=MaxVal(abs(v2-v1)) !convergence criterium: largest element in abs(v2-v1) .lt. 1e-7
      CALL SETTEXTPOSITION(3,2,curpos)
	WRITE(10,"('Iteration #=',I5)") t
	CALL SETTEXTPOSITION(4,2,curpos)
	WRITE(10,"('# of indices that have changed',I5)") dg
	CALL SETTEXTPOSITION(5,2,curpos)
	WRITE(10,"('Largest element in abs(v2-v1):',F20.8)") dv
	CALL SETTEXTPOSITION(7,2,curpos)
	WRITE(10,"('Number of consecutive runs with unchanged indices='
     + ,I3)") nof0
      v1=v2
      g1=g2
      t=t+1
      end do
      mini=MinVal(g1)
	i=MinVal(mini)
	maxi=MaxVal(g1)
	j=MaxVal(maxi)
	if (t>tmax) then
        CALL GETTEXTPOSITION(curpos)    
	  CALL SETTEXTPOSITION(curpos.row+2,2,curpos)
        WRITE(10,*) "Maximum number of iterations exceeded."
      endif
	if (i .eq. 1) then
	    CALL GETTEXTPOSITION(curpos)    
		CALL SETTEXTPOSITION(curpos.row+2,2,curpos)
          WRITE(10,"(A)") 'Policy function hits lower bound of grid'
      endif
	if (j .eq. n) then
	    CALL GETTEXTPOSITION(curpos)    
		CALL SETTEXTPOSITION(curpos.row+1,2,curpos)
          WRITE(10,"(A)") 'Policy function hits upper bound of grid'
      endif      
      g=g1
      End Subroutine