C Ramsey3b:
C
C Purpose: Solve for the policy function of the simple
C          stochastic Ramsey model with m-state Markov Chain
C          using value function iteration
C
C The parameters of the model are:
C
C alpha=elasticity of production with respect to capital,
C       must be in (0,1)
C beta =discount factor of utility function,
C       must be in (0,1)
C delta=rate of capital depreciation,
C       must be in (0,1]
C eta  =elasticity of marginal utility,
C       must be greater than zero
C
C n    =number of grid points

      use DFLIB
      use IMSLF90 

      Real(8) alpha, beta, delta, eta, rstar, epsilon
	Real(8) kmin, kmax, ku, ko, dk, size	
	REAL(8), ALLOCATABLE :: grid(:), z(:), p(:,:), pi(:,:), ks(:),
     +                        v0(:,:)
	REAL(4) Asec
	Integer(4) ngrid, m, stopafter
	Integer(4), ALLOCATABLE :: g(:,:)
	CHARACTER(256) RunTime
	CHARACTER(50) MyDate
	CHARACTER(20) Outfile
	CHARACTER(12) OutFormat	
	CHARACTER(12) InFormat
	CHARACTER(1) Inkey
	CHARACTER(2) Text
      Logical ok
      External u
      TYPE (rccoord) curpos      

      common /Par/ alpha, delta, eta
      common /DGE/ epsilon, stopafter

C Change Appearance of the program window
      ok=INITIALSETTINGS()
     
C Read Parameters from File
      Call GetD('alpha','Parameters.txt',alpha)
	Call GetD('eta','Parameters.txt',eta)      
	Call GetD('delta','Parameters.txt',delta)      
	Call GetD('rstar','Parameters.txt',rstar)
	Call GetD('epsilon','Parameters.txt',epsilon)
	Call GetD('kmin','Parameters.txt',kmin)
	Call GetD('kmax','Parameters.txt',kmax)
	Call GetI('m','Parameters.txt',m)
	Call GetI('ngrid','Parameters.txt',ngrid)
	Call GetI('stopafter','Parameters.txt',stopafter)
	Call GetC('outfile','Parameters.txt',outfile)      

C Compute derived parameters	
      beta=1/rstar
      
      WRITE(OutFormat,"(A1,I2,A)") '(', m, '(I5,2X))'
	WRITE(InFormat,"(A1,I2,A5)") '(',m,'F5.2)'

C Allocate Space
      ALLOCATE(grid(1:ngrid),g(1:ngrid,1:ngrid),p(1:m,1:m), pi(1:m,1:m),
     +        ks(1:m),z(1:m), v0(1:ngrid,1:m))


C Read z and p
      open(10,action="Read",file='Parameters.txt',err=105)
	ok=.FALSE.
	Do while ((.not. eof(10)) .and. (.not. ok))
	    Read(10,'(A2)') Text	    
	    IF (Text .eq. 'z=') then
	       ok=.TRUE.
	       read(10,InFormat) z(1:m)
	    endif
      end do
	if (.not. ok) then
	I3=MESSAGEBOXQQ('z not found',"Error",MB$OK)
	goto 105
      endif
      Rewind(10)
      ok=.FALSE.
	Do while ((.not. eof(10)) .and. (.not. ok))
	    Read(10,'(A2)') Text	    
	    IF (Text .eq. 'p=') then
	       ok=.TRUE.
	       do i=1,m
	         read(10,InFormat) p(i,1:m)
	       end do
	    endif
      end do
	if (.not. ok) then
	I3=MESSAGEBOXQQ('p not found',"Error",MB$OK)
	goto 105
      endif

C Create Output Window
      Call MakeSmallWindow(10,"Output Window",2)      			

	
C Compute Sequence of stationary capital stocks and
C intial value function
      dk=0.0
      Do j=1,m
         ks(j)=((alpha*beta*z(j))/(1-beta*(1-delta)))**(1/(1-alpha))
         do i=1,ngrid
	      v0(i,j)=u(z(j),ks(j),ks(j))
	   end do
	End Do
   	pi=eye(m)-beta*Transpose(p)

	Call DLINRG(m,pi,m,pi,m)
      v0=MatMul(v0,pi)
   
C Compute a grid over  [k1,kn] 
      ku=kmin*ks(1)
      ko=kmax*ks(m)
      dk=DBLE((ko-ku)/(ngrid-1))

      CALL SeqaD(ku,dk,ngrid,grid)


      ASec=SECNDS(0.0)
C Call routine that solves for the policy function       
      Call SolveVI(u,beta,grid,z,p,ngrid,m,v0,g)
      
	CALL ElapsedTime(SECNDS(0.0)-Asec,RunTime)		
	CALL GETTEXTPOSITION(curpos)
	CALL SETTEXTPOSITION(curpos.row+1,2,curpos)
	WRITE(10,"(A100)") RunTime

     

C Find stationary solutions and compare to exact one
      Do j=1,m
	i=1
	Do while (g(i,j) .ne. g(g(i,j),j))	  
	  i=g(i,j)	
	End Do
	  i=g(i,j)
        CALL GETTEXTPOSITION(curpos)
	  CALL SETTEXTPOSITION(curpos.row+1,2,curpos)
	  WRITE(10,"(A3,I2,A1,I4,A2,F10.5)") 'ks(', j, ',',i,')=',
     +	   (grid(i)/ks(j))
      End Do
	
C Write solution to file
      open(11,file=Outfile)
	WRITE(11,"(A50)") MyDate()
	WRITE(11,"(A100)") RunTime
      Do i=1,ngrid	   
	     WRITE(11,OutFormat) g(i,1:m)	   
      end do

	close(11)

105   if (.not. ok) then      
	I3=MESSAGEBOXQQ('File not found',"Error",MB$OK)
	End If
 
      END

	Real(8) Function u(z,k1,k2)

	Real(8) alpha, delta, eta
	Real(8) z, k1, k2 

	Common /Par/alpha, delta, eta
      
	u=z*(k1**alpha)+(1-delta)*k1-k2

	if (u .lt. 0.0) then
	   u=-1/0.0
      else
	   if (eta .eq. 1.0) then
	      u=log(u)
	   else
	      u=(u**(1-eta))/(1-eta)
	   endif
	endif

	Return

      End Function



