C Ramsey4a:
C
C Purpose: Simulate the prototype real business cycle
C          model. The solution procedure is 
C          extension of deterministic path.
C
C The parameters of the model are:
C
C alpha=elasticity of production with respect to capital,
C       must be in (0,1)
C rstar=1+real rate of interest
C delta=rate of capital depreciation,
C       must be in (0,1)
C eta  =elasticity of marginal utility,
C       must be greater than zero
C nstar=share of time devoted to work
C rho  =autoregressive parameter of productivity shock
C simga=standard deviation of innovations to productivity shock
C tmax =number of periods considered for convergence
C
C nobs =length of individual time series
C nofs =number of simulations

C First, we define common parameters and vector in a module
C This Module defines parameters and vectors common
C to the main program unit as well as to Sys1.
      Module Par

      Integer(4) Tmax
      Real(8) a, alpha, beta, delta, eta, theta, k0, kstar
      Real(8), Allocatable :: z2(:)
    
      END Module Par

C Here comes the code for the main program unit
      Program Ramsey4a

      use Par
      use DFLIB
      use IMSLF90 

      Real(8)  nstar, rho, sigma, errrel,Fnorm
      Integer(4) nobs, nofs, npars, itmax

      Real(8) yk, ck, yc, kn, cstar, istar, wstar, ystar,xmean(4),
     +        cov(5,5)
      Real(8) zmax, kmax, Lambda, moments(5,3)
      Real(8), allocatable :: zt(:), eps(:), kt(:), ct(:), yt(:), wt(:),
     +                        it(:), nt(:), bounds(:,:), x1(:), x2(:),
     +                        xt(:,:), xh(:,:)
      REAL(4) Asec
      Integer(4) t, s,vo,nmiss,sumwt
        
      CHARACTER(256) RunTime
      CHARACTER(50) MyDate
      CHARACTER(15) Vars(5)
      CHARACTER(20) OutFile
      Logical ok
      External Sys1, Sys2

      TYPE (rccoord) curpos      

      DATA moments /15*0.0/
      DATA Vars(1)/'Output'/
      DATA Vars(2)/'Investment'/
      DATA Vars(3)/'Consumption'/
      DATA Vars(4)/'Hours'/
      DATA Vars(5)/'Real Wage'/

C Read Parameters from File
      Call GetD('a','Parameters.txt',a)
      Call GetD('alpha','Parameters.txt',alpha)
      Call GetD('eta','Parameters.txt',eta)
      Call GetD('delta','Parameters.txt',delta)
      Call GetD('nstar','Parameters.txt',nstar)
      Call GetD('beta','Parameters.txt',beta)
      Call GetD('rho','Parameters.txt',rho)
      Call GetD('sigma','Parameters.txt',sigma)
      Call GetD('lambda','Parameters.txt',lambda)
      Call GetI('nobs','Parameters.txt',nobs)
      Call GetI('nofs','Parameters.txt',nofs)
      Call GetI('tmax','Parameters.txt',tmax)
      Call GetC('outfile','Parameters.txt',outfile)

      npars=2*tmax-1

C Compute derived parameters and stationary solution
      yk=((a**eta)-beta*(1-delta))/(beta*(1-alpha)) ! output-capital ratio      
      ck=yk+(1-delta-a)                             ! consumption-capital ratio 
      yc=yk/ck                                      ! inverse of average propensity to consume 
      theta=(alpha*yc*(1-nstar))/nstar              ! theta implied by nstar          
      kn=yk**(-1.0/alpha)                           ! capital-labor ratio             
      kstar=kn*nstar                                ! stationary capital stock k=K/A  
      istar=(a+delta-1)*kstar                       ! stationary level of investment  
      cstar=yk*kstar - istar                        ! stationary level of consumption 
      ystar=yk*kstar                                ! stationary level of output
      wstar=alpha*(nstar**(alpha-1))*(kstar**(1-alpha)) ! stationary level of real wage

C Change Appearance of the program window
      ok=INITIALSETTINGS()

C Create Output Window
      Call MakeSmallWindow(10,"Output Window",2)            

C Check the strict concavity of the one-period utility function
      if (eta .lt. (theta/(1+theta))) then
         CALL SETTEXTPOSITION(2,2,curpos)
         WRITE(10,"(A)") 'One-period utility function is not concave'
	   CALL SETTEXTPOSITION(3,2,curpos) 
	   WRITE(10,"(A,F6.2)") 'eta= ', eta
	   CALL SETTEXTPOSITION(4,2,curpos)
	   WRITE(10,"(A,F6.2)") 'theta/(1+theta) = ', (theta/(1+theta))
	   CALL SETTEXTPOSITION(5,2,curpos)
	   WRITE(10,"(A)") 'Choose a larger value of eta'
         CALL SETTEXTPOSITION(6,2,curpos)
	   WRITE(10,"(A)") 'Program stops'
	   goto 4000
	end if
      

C Allocate space
      ALLOCATE(eps(1:nobs),zt(1:nobs),kt(1:nobs+1),ct(1:nobs),
     +       yt(1:nobs),it(1:nobs),nt(1:nobs), bounds(1:NPars,1:2),
     +         z2(1:tmax), x1(1:Npars), x2(1:NPars), xt(1:nobs,1:5),
     +         xh(1:nobs-1,1:2), wt(1:nobs))

C Start
      ASEC=Secnds(0.0)      
      Errrel=0.0001
	itmax=200

C Here begins the outer loop that iterates over
C the number of simulations specified in nofs

      Do s=1,nofs
      CALL SETTEXTPOSITION(2,2,curpos)
      WRITE(10,"('Simulation #=',I4)") s

C Draw shocks from a standard normal distribution       
      CALL DRNNOR(nobs, eps)    

C Compute the time series of the productivity shock            
      ZT(1)=exp(Sigma*eps(1))

      Do I=1,(nobs-1)
       ZT(I+1)=(ZT(I)**Rho)*exp(Sigma*eps(i))             
      End Do

C Compute upper bound for capital stock
      zmax=zt(IDMAX(nobs,zt,1))
      kmax=(delta/zmax)**(-1/alpha)

C Compute matrix of bounds and starting values
      Do I=1,tmax-1
       x1(I)=Kstar
         bounds(i,1)=0.0
       bounds(i,2)=kmax
      End Do
      Do I=1,Tmax
       x1(tmax-1+i)=nstar
       bounds(tmax-1+i,1)=0.0
       bounds(tmax-1+i,2)=1.0
      End Do
      kt(1)=kstar
      Do t=1,nobs
      CALL SETTEXTPOSITION(4,2,curpos)
      WRITE(10,"('Period #=',I5)") t
       k0=kt(t)         ! set initial value of k equal to the previous stock of capital 
       z2(1)=zt(t)      ! compute the expected time path of productivity shock from the current shock onward 
       Do I=1,Tmax-1
          z2(i+1)=z2(i)**Rho
       End Do
      Call FixvMN(Npars,x1,bounds,Sys1,x2)
      
C	Call DNEQNF(Sys2,Errrel,Npars,Itmax,x1,x2,Fnorm)
          
       kt(t+1)=x2(1)
       it(t)  =a*kt(t+1)+(delta-1)*kt(t)
       nt(t)  =x2(tmax)
       yt(t)  =zt(t)*(nt(t)**alpha)*(kt(t)**(1-alpha))
       ct(t)  =yt(t) - it(t)
       wt(t)  = alpha*zt(t)*(nt(t)**(alpha-1))*(kt(t)**(1-alpha))

C Compute percentage deviations from trend
       yt(t)=dlog(yt(t))
       it(t)=dlog(it(t))
       ct(t)=dlog(ct(t))
       nt(t)=dlog(nt(t))
       wt(t)=dlog(wt(t))
          x1=x2        ! use old solution as guess for the next one 

      End Do

C Filtering and computation of correlations and standard deviations
      
      CALL HPFilter(Nobs,yt,Lambda,xt(1:nobs,1))
      CALL HPFilter(Nobs,it,Lambda,xt(1:nobs,2))
      CALL HPFILTER(Nobs,ct,Lambda,xt(1:nobs,3))
      CALL HPFilter(Nobs,nt,Lambda,xt(1:nobs,4))
      CALL HPFilter(Nobs,wt,Lambda,xt(1:nobs,5))


C   CALL DCORVC(0,nobs,4,xt,nobs,0,0,0,3,xmean,cov,4,vo,1,nobs,
C     +            nmiss,sumwt)
      CALL Corrx(nobs,5,xt,cov)

      moments(1:5,1)=moments(1:5,1)+100*Diagonals(cov)
      moments(2:5,2)=moments(2:5,2)+cov(1,2:5)
    
C Autocorrelations
      Do I=1,5
       xh(1:nobs-1,1)=xt(1:nobs-1,i)
       xh(1:nobs-1,2)=xt(2:nobs,i)      
       CALL Corrx(nobs-1,2,xh,cov(1:2,1:2))
         moments(I,3)=moments(I,3)+cov(1,2)
      End Do

      End Do
    
      Moments=moments/nofs
      Moments(1,2)=1


      CALL ElapsedTime(SECNDS(0.0)-Asec,RunTime)      
      CALL GETTEXTPOSITION(curpos)
      CALL SETTEXTPOSITION(curpos.row+1,2,curpos)
      WRITE(10,"(A100)") RunTime
    
C Write solution to file
      open(11,file=trim(OutFile))
      WRITE(11,"(A50)") MyDate()
      WRITE(11,"(A100)") RunTime
      Do i=1,5 
         WRITE(11,"(A15,3(F7.2,3X))") Vars(I), moments(I,1:3)
      end do
	WRITE(11,"(A)") 'Column 1: Variable Name'
      WRITE(11,"(A)") 'Column 2: Standard Deviation'
      WRITE(11,"(A)") 'Column 3: Cross Correlation with Output'
      WRITE(11,"(A)") 'Column 4: First Order Autocorrelation'
      WRITE(11,"(A)") ' '      
      WRITE(11,"(A)") 'Parameter values:'
      WRITE(11,"('a=       ',F8.4)") a
      WRITE(11,"('alpha=   ',F8.4)") alpha
      WRITE(11,"('beta=    ',F8.4)") beta 
      WRITE(11,"('nstar=   ',F8.4)") nstar
      WRITE(11,"('delta=   ',F8.4)") delta
      WRITE(11,"('rho=     ',F8.4)") rho
      WRITE(11,"('sigma=   ',F8.4)") sigma
      WRITE(11,"('tmax=    ',I4)")  tmax
      WRITE(11,"('nobs=    ',I4)")   nobs
      WRITE(11,"('nofs=    ',I4)")   nofs

      close(11)
 
4000  END

      Subroutine Sys1(mpar,npar,x,fx)

      use Par
      Integer mpar, npar
      Real(8) x(1:npar), fx(1:mpar)

      Integer(4) I, J
      Real(8) c1, c2
      Real(8) kt(1:tmax-1), nt(1:tmax)
             
       kt=x(1:Tmax-1)
         nt=x(Tmax:2*Tmax-1)
         
   
         c1=z2(1)*(nt(1)**alpha)*(k0**(1-alpha))+(1-delta)*k0 - a*kt(1)
         c2=z2(2)*(nt(2)**alpha)*(kt(1)**(1-alpha)) + (1-delta)*kt(1)
     +      - a*kt(2)
       fx(1)= theta*c1 - alpha*(1-nt(1))*z2(1)*(nt(1)**(alpha-1))
     +        *(k0**(1-alpha))
       fx(2)=((c2/c1)**eta)*(((1-nt(1))/(1-nt(2)))**(theta*(1-eta)))-
     +    beta*(a**(-eta))*(1-delta+(1-alpha)*z2(2)*(nt(2)**alpha)*
     +   (kt(1)**(-alpha)))
   
      J=3
      Do I=2,(Tmax-2)
   
         c1=c2
         c2=z2(i+1)*(nt(i+1)**alpha)*(kt(i)**(1-alpha)) 
     +     + (1-delta)*kt(i) - a*kt(i+1)
        fx(j)=theta*c1-alpha*(1-nt(i))*z2(i)*(nt(i)**(alpha-1))*
     +        (kt(i-1)**(1-alpha))
      fx(j+1)=((c2/c1)**eta)*(((1-nt(i))/(1-nt(i+1)))**(theta*(1-eta)))
     +        -beta*(a**(-eta))*(1-delta+(1-alpha)*z2(i+1)*
     +        (nt(i+1)**alpha)*(kt(i)**(-alpha)))
           j=j+2
      End Do
         c1=c2;
         c2=z2(i+1)*(nt(i+1)**alpha)*(kt(i)**(1-alpha)) 
     +      + (1-delta)*kt(i) - a*kstar
      fx(j)=theta*c1-alpha*(1-nt(i))*z2(i)*(nt(i)**(alpha-1))*
     +      (kt(i-1)**(1-alpha))
      fx(j+1)=((c2/c1)**eta)*(((1-nt(i))/(1-nt(i+1)))**(theta*(1-eta)))
     +      -beta*(a**(-eta))*(1-delta+(1-alpha)*z2(i+1)*
     +      (nt(i+1)**alpha)*(kt(i)**(-alpha)))
      fx(j+2)=theta*c2-alpha*(1-nt(i+1))*z2(i+1)*(nt(i+1)**(alpha-1))*
     +       (kt(i)**(1-alpha))

      End Subroutine


      Subroutine Sys2(x,fx,npar)

      use Par
      Integer npar
      Real(8) x(1:npar), fx(1:npar)

      Integer(4) I, J
      Real(8) c1, c2
      Real(8) kt(1:tmax-1), nt(1:tmax)
             
         kt=x(1:Tmax-1)
         nt=x(Tmax:2*Tmax-1)
         
   
         c1=z2(1)*(nt(1)**alpha)*(k0**(1-alpha))+(1-delta)*k0 - a*kt(1)
         c2=z2(2)*(nt(2)**alpha)*(kt(1)**(1-alpha)) + (1-delta)*kt(1)
     +      - a*kt(2)
       fx(1)= theta*c1 - alpha*(1-nt(1))*z2(1)*(nt(1)**(alpha-1))
     +        *(k0**(1-alpha))
       fx(2)=((c2/c1)**eta)*(((1-nt(1))/(1-nt(2)))**(theta*(1-eta)))-
     +    beta*(a**(-eta))*(1-delta+(1-alpha)*z2(2)*(nt(2)**alpha)*
     +   (kt(1)**(-alpha)))
   
      J=3
      Do I=2,(Tmax-2)
   
         c1=c2
         c2=z2(i+1)*(nt(i+1)**alpha)*(kt(i)**(1-alpha)) 
     +     + (1-delta)*kt(i) - a*kt(i+1)
        fx(j)=theta*c1-alpha*(1-nt(i))*z2(i)*(nt(i)**(alpha-1))*
     +        (kt(i-1)**(1-alpha))
      fx(j+1)=((c2/c1)**eta)*(((1-nt(i))/(1-nt(i+1)))**(theta*(1-eta)))
     +        -beta*(a**(-eta))*(1-delta+(1-alpha)*z2(i+1)*
     +        (nt(i+1)**alpha)*(kt(i)**(-alpha)))
           j=j+2
      End Do
         c1=c2;
         c2=z2(i+1)*(nt(i+1)**alpha)*(kt(i)**(1-alpha)) 
     +      + (1-delta)*kt(i) - a*kstar
      fx(j)=theta*c1-alpha*(1-nt(i))*z2(i)*(nt(i)**(alpha-1))*
     +      (kt(i-1)**(1-alpha))
      fx(j+1)=((c2/c1)**eta)*(((1-nt(i))/(1-nt(i+1)))**(theta*(1-eta)))
     +      -beta*(a**(-eta))*(1-delta+(1-alpha)*z2(i+1)*
     +      (nt(i+1)**alpha)*(kt(i)**(-alpha)))
      fx(j+2)=theta*c2-alpha*(1-nt(i+1))*z2(i+1)*(nt(i+1)**(alpha-1))*
     +       (kt(i)**(1-alpha))

      End Subroutine

