C Ramsey4b:
C
C Purpose: Simulate the prototype real business cycle
C          model. The solution procedure is 
C          iteration over the value function
C
C The parameters of the model are:
C
C alpha=elasticity of production with respect to capital,
C       must be in (0,1)
C rstar=1+real rate of interest
C delta=rate of capital depreciation,
C       must be in (0,1)
C eta  =elasticity of marginal utility,
C       must be greater than zero
C nstar=share of time devoted to work
C rho  =autoregressive parameter of productivity shock
C simga=standard deviation of innovations to productivity shock
C size= width of the grid that approximates the continuous
C       Markov process by a Markov chain
C    m= number of possible realizations of the Markov Chain
C ngrid=number of grid points for the capital stock
C
C nobs =length of individual time series
C nofs =number of simulations

C First, we define common parameters and vector in a module
C This Module defines parameters and vectors common
C to the main program unit as well as to Sys1.
      Module Pars
   
      Real(8) a, alpha, delta, eta, theta, nstar  
    
      END Module Pars

C Here comes the code for the main program unit
      Program Ramsey4b

      use Pars
      use DFLIB
      use IMSLF90 

      Real(8) beta, rho, sigma, size, epsilon
      Integer(4) nobs, nofs, m, ngrid, stopafter
      Integer(4), allocatable :: g(:,:), k(:), zindex(:)

      Real(8) yk, ck, yc, cstar, kstar, istar, ystar,xmean(5),
     +        wstar, cov(5,5), kn
      Real(8) zmax, zmin, kmax, kmin, Lambda, moments(5,3), k1, k2, dk
      Real(8), allocatable :: zt(:), eps(:), kt(:), ct(:), yt(:),
     +                        it(:), nt(:), grid(:), z(:), p(:,:),
     +                        xt(:,:), xh(:,:), v0(:,:),  pi(:,:),
     +                        wt(:)
      REAL(4) Asec
      Integer(4) t, itn, i0, i, j, pos
    
      CHARACTER(256) RunTime
      CHARACTER(50) MyDate
      CHARACTER(15) Vars(5)
      CHARACTER(20) OutFile
      Logical ok
      External u, GetN

      Common /DGE/ epsilon, stopafter

      TYPE (rccoord) curpos      

      DATA moments /15*0.0/
      DATA Vars(1)/'Output'/
      DATA Vars(2)/'Investment'/
      DATA Vars(3)/'Consumption'/
      DATA Vars(4)/'Hours'/
      DATA Vars(5)/'Real Wage'/

C Read Parameters from File
      Call GetD('a','Parameters.txt',a)
      Call GetD('alpha','Parameters.txt',alpha)
      Call GetD('eta','Parameters.txt',eta)
      Call GetD('delta','Parameters.txt',delta)
      Call GetD('nstar','Parameters.txt',nstar)
      Call GetD('beta','Parameters.txt',beta)
      Call GetD('rho','Parameters.txt',rho)
      Call GetD('sigma','Parameters.txt',sigma)
      Call GetD('lambda','Parameters.txt',lambda)
      Call GetD('size','Parameters.txt',size)
      Call GetD('epsilon','Parameters.txt',epsilon)
      Call GetD('kmax','Parameters.txt',kmax)
      Call GetD('kmin','Parameters.txt',kmin)
      Call GetI('nobs','Parameters.txt',nobs)
      Call GetI('nofs','Parameters.txt',nofs)
      Call GetI('m','Parameters.txt',m)
      Call GetI('ngrid','Parameters.txt',ngrid)
      Call GetI('stopafter','Parameters.txt',stopafter)
      Call GetC('outfile','Parameters.txt',outfile)      

C Compute derived parameters and stationary solution      

      yk=((a**eta)-beta*(1-delta))/(beta*(1-alpha)) ! output-capital ratio      
      ck=yk+(1-delta-a)                             ! consumption-capital ratio 
      yc=yk/ck                                      ! inverse of average propensity to consume 
      theta=(alpha*yc*(1-nstar))/nstar              ! theta implied by nstar          
      kn=yk**(-1.0/alpha)                           ! capital-labor ratio             
      kstar=kn*nstar                                ! stationary capital stock k=K/A  
      istar=(a+delta-1)*kstar                       ! stationary level of investment  
      cstar=yk*kstar - istar                        ! stationary level of consumption 
      ystar=yk*kstar                                ! stationary level of output
      wstar=alpha*(nstar**(alpha-1))*(kstar**(1-alpha)) ! stationary level of real wage

C Change Appearance of the program window
      ok=INITIALSETTINGS()

C Create Output Window
      Call MakeSmallWindow(10,"Output Window",2)            
      
C Check the strict concavity of the one-period utility function
      if (eta .lt. (theta/(1+theta))) then
         CALL SETTEXTPOSITION(2,2,curpos)
         WRITE(10,"(A)") 'One-period utility function is not concave'
	   CALL SETTEXTPOSITION(3,2,curpos) 
	   WRITE(10,"(A,F6.2)") 'eta= ', eta
	   CALL SETTEXTPOSITION(4,2,curpos)
	   WRITE(10,"(A,F6.2)") 'theta/(1+theta) = ', (theta/(1+theta))
	   CALL SETTEXTPOSITION(5,2,curpos)
	   WRITE(10,"(A)") 'Choose a larger value of eta'
         CALL SETTEXTPOSITION(6,2,curpos)
	   WRITE(10,"(A)") 'Program stops'
	   goto 4000
	end if

      

C Allocate space
      ALLOCATE(eps(1:nobs),zt(1:nobs),kt(1:nobs+1),ct(1:nobs),
     +       yt(1:nobs),it(1:nobs),nt(1:nobs),
     +         z(1:m), xt(1:nobs,1:5), p(1:m,1:m), grid(1:ngrid),
     +         g(1:ngrid,1:m),k(1:nobs+1),zindex(1:nobs),
     +         xh(1:nobs-1,1:2),v0(1:ngrid,1:m),pi(1:m,1:m),wt(1:nobs))

C Start
      ASEC=Secnds(0.0)      

C Get Markov Chain Approximation
      CALL MarkovAR(size,m,rho,sigma,z,p)
      z=exp(z)
C Compute grid for capital stock
      zmax=maxval(z)
      zmin=minval(z)

      k1=kmax*nstar*(zmax**(1/alpha))*(yk**(-1.0/alpha))    
      k2=kmin*nstar*(zmin**(1/alpha))*(yk**(-1.0/alpha))  
      
      dk=DBLE((k1-k2)/(ngrid-1))

      CALL SeqaD(k2,dk,ngrid,grid)

C Compute initial value function

      Do i=1,ngrid       
       Do j=1,m
            dk=nstar*(z(j)**(1/alpha))*(yk**(-1.0/alpha))
          v0(i,j)=u(z(j),dk,dk)
       End Do
      End Do
      pi=eye(m)-beta*(a**(1-eta))*Transpose(p)
      Call DLINRG(m,pi,m,pi,m)
      v0=MatMul(v0,pi)
     
C Compute policy function
      ASec=SECNDS(0.0)
      
      Call SolveVI(u,beta*(a**(1-eta)),grid,z,p,ngrid,m,v0,g)
    
C     open(11,File='G.txt')
C   Do i=1,ngrid
C      WRITE(11,"(I5,2X,3(I5,2X))") I, g(i,1:m)
C   end do
C   close(11)

C Simulate the model
      I=1
      do while (grid(i) .le.  kstar)
       i=i+1
      end do
      k(1)=i
      kt(1)=grid(k(1))  ! always start at stationary solution

      CALL GETTEXTPOSITION(curpos)
      CALL SETTEXTPOSITION(curpos.row+1,2,curpos)
      WRITE(10,"(A)") 'Simulate Model'
      CALL GETTEXTPOSITION(curpos)
      pos=curpos.row+1
      i0=(m/2)+1
      Do itn=1,nofs    
       CALL SETTEXTPOSITION(pos,2,curpos)
       WRITE(10,"('Simulation #',I3)") itn
       CALL MarkovTP(nobs,m,p,i0,zindex)       
         do i=1,nobs
           k(i+1)=g(k(i),zindex(i))            
         kt(i+1)=grid(k(i+1))
         zt(i)=z(zindex(i))
         end do
        
C Compute output, investment, hours and consumption 
        do i=1,nobs              
        Call GetCN(kt(i),kt(i+1),zt(i),ct(i),nt(i))   
         yt(i)=zt(i)*(nt(i)**alpha)*(kt(i)**(1-alpha))         
         it(i)=a*kt(i+1)+(delta-1)*kt(i)
       wt(i)=alpha*zt(i)*(nt(i)**(alpha-1))*(kt(i)**(1-alpha))

C Compute percentage deviations from trend
         yt(i)=dlog(yt(i)) !yt(i)=(yt(i)-ystar)/ystar
         it(i)=dlog(it(i)) !it(i)=(it(i)-istar)/istar
         ct(i)=dlog(ct(i)) !ct(i)=(ct(i)-cstar)/cstar
         nt(i)=dlog(nt(i)) !nt(i)=(nt(i)-nstar)/nstar
         wt(i)=dlog(wt(i)) !wt(i)=(wt(i)-wstar)/wstar
      End Do

C Filtering and computation of correlations and standard deviations
      
      CALL HPFilter(Nobs,yt,Lambda,xt(1:nobs,1))
      CALL HPFilter(Nobs,it,Lambda,xt(1:nobs,2))
      CALL HPFILTER(Nobs,ct,Lambda,xt(1:nobs,3))
      CALL HPFilter(Nobs,nt,Lambda,xt(1:nobs,4))
      CALL HPFilter(Nobs,wt,Lambda,xt(1:nobs,5))
      CALL Corrx(nobs,5,xt,cov)

      moments(1:5,1)=moments(1:5,1)+100*Diagonals(cov)
      moments(2:5,2)=moments(2:5,2)+cov(1,2:5)
    
C Autocorrelations
      Do I=1,5
       xh(1:nobs-1,1)=xt(1:nobs-1,i)
       xh(1:nobs-1,2)=xt(2:nobs,i)      
       CALL Corrx(nobs-1,2,xh,cov(1:2,1:2))
         moments(I,3)=moments(I,3)+cov(1,2)
      End Do
   
      End Do

      Moments=moments/nofs
      Moments(1,2)=1


      CALL ElapsedTime(SECNDS(0.0)-Asec,RunTime)      
      CALL GETTEXTPOSITION(curpos)
      CALL SETTEXTPOSITION(curpos.row+1,2,curpos)
      WRITE(10,"(A100)") RunTime
    
C Write solution to file
      open(11,file=trim(OutFile))
      WRITE(11,"(A50)") MyDate()
      WRITE(11,"(A100)") RunTime
      Do i=1,5 
         WRITE(11,"(A15,3(F7.2,3X))") Vars(I), moments(I,1:3)
      end do    
      WRITE(11,"(A)") 'Column 1: Variable Name'
      WRITE(11,"(A)") 'Column 2: Standard Deviation'
      WRITE(11,"(A)") 'Column 3: Cross Correlation with Output'
      WRITE(11,"(A)") 'Column 4: First Order Autocorrelation'
      WRITE(11,"(A)") ' '
      WRITE(11,"(A)") 'Parameter values:'
      WRITE(11,"('a=       ',F8.4)") a
      WRITE(11,"('alpha=   ',F8.4)") alpha
      WRITE(11,"('beta=    ',F8.4)") beta 
      WRITE(11,"('nstar=   ',F8.4)") nstar
      WRITE(11,"('delta=   ',F8.4)") delta
      WRITE(11,"('rho=     ',F8.4)") rho
      WRITE(11,"('sigma=   ',F8.4)") sigma
      WRITE(11,"('epsilon= ',F8.4)") epsilon
      WRITE(11,"('size=    ',F8.4)") size
      WRITE(11,"('m=       ',I2)")   m
      WRITE(11,"('ngrid=   ',I4)")   ngrid
      WRITE(11,"('stopafter',I4)")   stopafter
      WRITE(11,"('nobs=    ',I4)")   nobs
      WRITE(11,"('nofs=    ',I4)")   nofs

      close(11)
 
4000  END

C ********************************************************************
C Subroutines
C
C u(z,k1,k2) returns the utility 
C
      Real(8) Function u(z,k1,k2)

      Use Pars
      
      Real(8) z, k1, k2

      Real(8) c, n, krit, z0, x1, x2, bounds(2)

      Real(8), External :: GetN

      Common /Help/ z0, x1, x2

      
      z0=z
      x1=k1
      x2=k2

      bounds(1)=0.0
      bounds(2)=1.0

C Check whether there is a solution for n
      krit=z*(k1**(1-alpha))+(1-delta)*k1 - a*k2
      if (krit .lt. 0.0) then
         c=-1/0  ! this returns -infinity
      else  ! compute n
         Call FixpMN(nstar,bounds,GetN,n)
          c=z*(n**alpha)*(k1**(1-alpha))+(1-delta)*k1-a*k2      
         if (c .lt. 0.0) then
            c=-1/0 ! this returns -infinity
       else
            if (eta .eq. 1.0) then
                c=dlog(c)+theta*dlog(1-n)
            else
                c=(c**(1-eta))*((1-n)**(theta*(1-eta)))
                c=c/(1-eta)
            endif
         endif
      endif
  
      u=c

      End Function


C GetCN returns c and n given k1, k2, and z

      Subroutine GetCN(k1,k2,z,c,n)

      use Pars

      Real(8) k1,k2, z, c, n, z0, x1, x2
      Real(8) bounds(2)
      Real(8), External :: GetN
      
      Common /Help/z0,x1,x2
  
      z0=z
      x1=k1
      x2=k2
      bounds(1)=0.0
      bounds(2)=1.0
  
      Call FixpMN(nstar,bounds,GetN,n)
      
      c=z*(n**alpha)*(k1**(1-alpha))+(1-delta)*k1-a*k2
  
      End Subroutine

C
      Subroutine GetN(I,n,fx)

      use Pars

      Integer I
      Real(8) n, fx
      Real(8) z0, x1, x2

      Common /Help/z0, x1, x2
  
 
      fx=theta*(z0*(n**alpha)*(x1**(1-alpha))+(1-delta)*x1 - a*x2) 
     + -  alpha*(1-n)*z0*(n**(alpha-1))*(x1**(1-alpha))
    
      End Subroutine
